var grpc = require('grpc');
var path = require('path')

var _responseProto = grpc.load(__dirname + '/protos/peer/proposal_response.proto').protos;
var _proposalProto = grpc.load(__dirname + '/protos/peer/proposal.proto').protos;
var _rwsetProto = grpc.load(__dirname + '/protos/ledger/rwset/rwset.proto').rwset;
var _kv_rwsetProto = grpc.load(__dirname + '/protos/ledger/rwset/kvrwset/kv_rwset.proto').kvrwset;
var _ccEventProto = grpc.load(__dirname + '/protos/peer/chaincode_event.proto').protos;
var _commonProto = grpc.load(__dirname + '/protos/common/common.proto').common;
var _identityProto = grpc.load(__dirname + '/protos/msp/identities.proto').msp;
var _chaincodeProto = grpc.load(__dirname + '/protos/peer/chaincode.proto').protos;

var _transProto = grpc.load(__dirname + '/protos/peer/transaction.proto').protos;

var BlockDecoderProxy = {}
BlockDecoderProxy.decodeProposalResponsePayload = function (proposal_response_payload_bytes) {
  var proposal_response_payload = {};
  var proto_proposal_response_payload = _responseProto.ProposalResponsePayload.decode(proposal_response_payload_bytes);
  proposal_response_payload.proposal_hash = proto_proposal_response_payload.getProposalHash().toBuffer().toString('hex');
  proposal_response_payload.extension = decodeChaincodeAction(proto_proposal_response_payload.getExtension());

  return proposal_response_payload;
}

BlockDecoderProxy.decodeProposal = function(proposal_bytes) {
  var proposal = {};
  var proto_proposal = _proposalProto.Proposal.decode(proposal_bytes);
  var header = _commonProto.Header.decode(proto_proposal.getHeader());
  proposal.header = decodeHeader(header)
  proposal.payload = decodeChaincodeProposalPayload(proto_proposal.getPayload())
  return proposal
}

BlockDecoderProxy.decodeEnvelopePayload = function(envelope_payload_bytes) {
  var envelope_payload = {};
  var proto_payload = _commonProto.Payload.decode(envelope_payload_bytes);
  envelope_payload.header = decodeHeader(proto_payload.getHeader());
  return envelope_payload
}


function decodeChaincodeAction(action_bytes) {
  var chaincode_action = {};
  var proto_chaincode_action = _proposalProto.ChaincodeAction.decode(action_bytes);
  chaincode_action.results = decodeReadWriteSets(proto_chaincode_action.getResults());
  chaincode_action.events = decodeChaincodeEvents(proto_chaincode_action.getEvents());
  chaincode_action.response = decodeResponse(proto_chaincode_action.getResponse());
  chaincode_action.chaincode_id = decodeChaincodeID(proto_chaincode_action.getChaincodeId());

  return chaincode_action;
}

function decodeReadWriteSets(rw_sets_bytes) {
  var proto_tx_read_write_set = _rwsetProto.TxReadWriteSet.decode(rw_sets_bytes);
  var tx_read_write_set = {};
  tx_read_write_set.data_model = proto_tx_read_write_set.getDataModel();
  if (proto_tx_read_write_set.getDataModel() === _rwsetProto.TxReadWriteSet.DataModel.KV) {
    tx_read_write_set.ns_rwset = [];
    let proto_ns_rwset = proto_tx_read_write_set.getNsRwset();
    for (let i in proto_ns_rwset) {
      let kv_rw_set = {};
      let proto_kv_rw_set = proto_ns_rwset[i];
      kv_rw_set.namespace = proto_kv_rw_set.getNamespace();
      kv_rw_set.rwset = decodeKVRWSet(proto_kv_rw_set.getRwset());
      tx_read_write_set.ns_rwset.push(kv_rw_set);
    }
  } else {
    // not able to decode this type of rw set, return the array of byte[]
    tx_read_write_set.ns_rwset = proto_tx_read_write_set.getNsRwset();
  }

  return tx_read_write_set;
}

function decodeKVRWSet(kv_bytes) {
  var proto_kv_rw_set = _kv_rwsetProto.KVRWSet.decode(kv_bytes);
  var kv_rw_set = {};

  // KV readwrite set has three arrays
  kv_rw_set.reads = [];
  kv_rw_set.range_queries_info = [];
  kv_rw_set.writes = [];

  // build reads
  let reads = kv_rw_set.reads;
  var proto_reads = proto_kv_rw_set.getReads();
  for (let i in proto_reads) {
    reads.push(decodeKVRead(proto_reads[i]));
  }

  // build range_queries_info
  let range_queries_info = kv_rw_set.range_queries_info;
  var proto_range_queries_info = proto_kv_rw_set.getRangeQueriesInfo();
  for (let i in proto_range_queries_info) {
    range_queries_info.push(decodeRangeQueryInfo(proto_range_queries_info[i]));
  }

  // build writes
  let writes = kv_rw_set.writes;
  var proto_writes = proto_kv_rw_set.getWrites();
  for (let i in proto_writes) {
    writes.push(decodeKVWrite(proto_writes[i]));
  }

  return kv_rw_set;
}

function decodeKVRead(proto_kv_read) {
  let kv_read = {};
  kv_read.key = proto_kv_read.getKey();
  let proto_version = proto_kv_read.getVersion();
  if (proto_version) {
    kv_read.version = {};
    kv_read.version.block_num = proto_version.getBlockNum().toString();
    kv_read.version.tx_num = proto_version.getTxNum().toString();
  } else {
    kv_read.version = null;
  }

  return kv_read;
}

function decodeRangeQueryInfo(proto_range_query_info) {
  let range_query_info = {};
  range_query_info.start_key = proto_range_query_info.getStartKey();
  range_query_info.end_key = proto_range_query_info.getEndKey();
  range_query_info.itr_exhausted = proto_range_query_info.getItrExhausted();

  // reads_info is one of QueryReads
  let proto_raw_reads = proto_range_query_info.getRawReads();
  if (proto_raw_reads) {
    range_query_info.raw_reads = {};
    range_query_info.raw_reads.kv_reads = [];
    for (let i in proto_raw_reads.kv_reads) {
      let kv_read = decodeKVRead(proto_raw_reads.kv_reads[i]);
      range_query_info.raw_reads.kv_reads.push(kv_read);
    }
  }
  // or QueryReadsMerkleSummary
  let proto_reads_merkle_hashes = proto_range_query_info.getReadsMerkleHashes();
  if (proto_reads_merkle_hashes) {
    range_query_info.reads_merkle_hashes = {};
    range_query_info.reads_merkle_hashes.max_degree = proto_reads_merkle_hashes.getMaxDegree();
    range_query_info.reads_merkle_hashes.max_level = proto_reads_merkle_hashes.getMaxLevel();
    range_query_info.reads_merkle_hashes.max_level_hashes = proto_reads_merkle_hashes.getMaxLevelHashes();
  }

  return range_query_info;
}

function decodeKVWrite(proto_kv_write) {
  let kv_write = {};
  kv_write.key = proto_kv_write.getKey();
  kv_write.is_delete = proto_kv_write.getIsDelete();
  kv_write.value = proto_kv_write.getValue().toBuffer().toString();

  return kv_write;
}

function decodeChaincodeEvents(event_bytes) {
  var events = {};
  var proto_events = _ccEventProto.ChaincodeEvent.decode(event_bytes);
  events.chaincode_id = proto_events.getChaincodeId();
  events.tx_id = proto_events.getTxId();
  events.event_name = proto_events.getEventName();
  events.payload = proto_events.getPayload().toBuffer();

  return events;
}

function decodeResponse(proto_response) {
  if (!proto_response) return null;
  var response = {};
  response.status = proto_response.getStatus();
  response.message = proto_response.getMessage();
  response.payload = proto_response.getPayload().toBuffer().toString();

  return response;
}

function decodeChaincodeID(proto_chaincode_id) {
  var chaincode_id = {};
  if(!proto_chaincode_id) {
    console.log('decodeChaincodeID - no proto_chaincode_id found');
    return chaincode_id;
  }
  chaincode_id.path = proto_chaincode_id.getPath();
  chaincode_id.name = proto_chaincode_id.getName();
  chaincode_id.version = proto_chaincode_id.getVersion();

  return chaincode_id;
}




function decodeVersion(version_long) {
  var version_string = version_long.toString();
  var version_int = Number.parseInt(version_string);

  return version_int;
}

function decodeHeader(proto_header) {
  var header = {};
  header.channel_header = decodeChannelHeader(proto_header.getChannelHeader().toBuffer());
  header.signature_header = decodeSignatureHeader(proto_header.getSignatureHeader().toBuffer());

  return header;
}

function decodeSignatureHeader(signature_header_bytes) {
  //logger.debug('decodeSignatureHeader - %s',signature_header_bytes);
  var signature_header = {};
  var proto_signature_header = _commonProto.SignatureHeader.decode(signature_header_bytes);
  signature_header.creator = decodeIdentity(proto_signature_header.getCreator().toBuffer());
  signature_header.nonce = proto_signature_header.getNonce().toBuffer();

  return signature_header;
}

function decodeIdentity(id_bytes) {
  //logger.debug('decodeIdentity - %s',id_bytes);
  var identity = {};
  try {
    var proto_identity = _identityProto.SerializedIdentity.decode(id_bytes);
    identity.Mspid = proto_identity.getMspid();
    identity.IdBytes = proto_identity.getIdBytes().toBuffer().toString();
  } catch (err) {
    logger.error('Failed to decode the identity: %s', err.stack ? err.stack : err);
  }

  return identity;
}


function decodeChannelHeader(header_bytes) {
  var channel_header = {};
  var proto_channel_header = _commonProto.ChannelHeader.decode(header_bytes);
  channel_header.type = proto_channel_header.getType();
  logger.debug('decodeChannelHeader - looking at type:%s',channel_header.type);
  channel_header.version = decodeVersion(proto_channel_header.getVersion());
  channel_header.timestamp = timeStampToDate(proto_channel_header.getTimestamp());
  channel_header.channel_id = proto_channel_header.getChannelId();
  channel_header.tx_id = proto_channel_header.getTxId();
  channel_header.epoch = proto_channel_header.getEpoch().toString(); //unit64
  //TODO need to decode this
  channel_header.extension = proto_channel_header.getExtension().toBuffer();

  return channel_header;
}


function decodeChaincodeProposalPayload(chaincode_proposal_payload_bytes) {
  var chaincode_proposal_payload = {};
  var proto_chaincode_proposal_payload = _proposalProto.ChaincodeProposalPayload.decode(chaincode_proposal_payload_bytes);
  chaincode_proposal_payload.input = decodeChaincodeProposalPayloadInput(proto_chaincode_proposal_payload.getInput());
  //TransientMap is not allowed to be included on ledger

  return chaincode_proposal_payload;
}

function decodeChaincodeProposalPayloadInput(chaincode_proposal_payload_input_bytes) {
  var chaincode_proposal_payload_input = {};

  // For a normal transaction, input is ChaincodeInvocationSpec.
  var proto_chaincode_invocation_spec = _chaincodeProto.ChaincodeInvocationSpec.decode(chaincode_proposal_payload_input_bytes);
  chaincode_proposal_payload_input.chaincode_spec = decodeChaincodeSpec(proto_chaincode_invocation_spec.getChaincodeSpec().toBuffer());

  return chaincode_proposal_payload_input;
}

const chaincode_type_as_string = {
  0: 'UNDEFINED',
  1: 'GOLANG',
  2: 'NODE',
  3: 'CAR',
  4: 'JAVA'
};

function chaincodeTypeToString(type) {
  let type_str = chaincode_type_as_string[type];
  if (typeof type_str == 'undefined') {
    return 'UNKNOWN';
  } else {
    return type_str;
  }
}

function decodeChaincodeSpec(chaincode_spec_bytes) {
  var chaincode_spec = {};
  var proto_chaincode_spec = _chaincodeProto.ChaincodeSpec.decode(chaincode_spec_bytes);
  chaincode_spec.type = proto_chaincode_spec.getType();
  // Add a string for the chaincode type (GOLANG, NODE, etc.)
  chaincode_spec.typeString = chaincodeTypeToString(chaincode_spec.type);
  chaincode_spec.input = decodeChaincodeInput(proto_chaincode_spec.getInput().toBuffer());
  chaincode_spec.chaincode_id = proto_chaincode_spec.getChaincodeId();
  chaincode_spec.timeout = proto_chaincode_spec.getTimeout();

  return chaincode_spec;
}

function decodeChaincodeInput(chaincode_spec_input_bytes) {
  var input = {};
  var proto_chaincode_input = _chaincodeProto.ChaincodeInput.decode(chaincode_spec_input_bytes);
  var args = proto_chaincode_input.getArgs();

  input.args = [];
  for (let i in args) {
    input.args.push(args[i].toBuffer());
  }
  let decorations = proto_chaincode_input.getDecorations();
  let keys = Object.keys(decorations.map);
  input.decorations = {};
  for (let i in keys) {
    input.decorations[keys[i]] = decorations.map[keys[i]].value.toBuffer();
  }

  return input;
}

function timeStampToDate(time_stamp) {
  if(!time_stamp) {
    return 'null';
  }
  var millis = time_stamp.seconds * 1000 + time_stamp.nanos / 1000000;
  var date = new Date(millis);

  return date.toString();
}


module.exports = BlockDecoderProxy
