const assert = require("assert");
const util = require('util');
const should = require('should');
const Promise = require('bluebird');

global.logger = console;

const ClientProxy = require("../lib/ClientProxy.js");

let req = {
  channel_id: 'channel01',
  chaincodeId: 'testcc0x1',
  chaincodeVersion: "v1.5",
  chaincodeType: 'golang',
  chaincodePath: "svn/projects/finchain/sourcecode/finchain-baas/chaincode/finchain",
  isUpdate: false,
  endorsementPolicy:{
    identities: [
      { role: { name: "member", mspId: "banksMSP" }},
      { role: { name: "admin", mspId: "platformMSP" }}
    ],
    policy: {
      "1-of": [{ "signed-by": 0 }, { "signed-by": 1 }]
    }
  }
};
let target_obj = {
  peer_url: "grpcs://peer1.banks.bankalliance.org:7051",
  peer_url_1: "grpcs://peer0.banks.bankalliance.org:7051",
  orderer_url: "grpcs://orderer0.bankalliance.org:7050",
  event_url: "grpcs://peer1.banks.bankalliance.org:7053",
  peer_opts: {
    pem: `-----BEGIN CERTIFICATE-----
MIICVjCCAfygAwIBAgIRAOEcfzFoQFonPveu3+K+E1IwCgYIKoZIzj0EAwIwfDEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xHzAdBgNVBAoTFmJhbmtzLmJhbmthbGxpYW5jZS5vcmcxJTAjBgNVBAMTHHRs
c2NhLmJhbmtzLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgwOTIwMTAxNjA4WhcNMjgw
OTE3MTAxNjA4WjB8MQswCQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREw
DwYDVQQHEwhzaGVuemhlbjEfMB0GA1UEChMWYmFua3MuYmFua2FsbGlhbmNlLm9y
ZzElMCMGA1UEAxMcdGxzY2EuYmFua3MuYmFua2FsbGlhbmNlLm9yZzBZMBMGByqG
SM49AgEGCCqGSM49AwEHA0IABJ3huUWMOaatLphxG2pEn5XCiOYO5QCBYnFLbVUA
SceWsx4eGn1zt7cEVDBtdCybMnyIAWLR4bOgn3WNm3T/5NCjXzBdMA4GA1UdDwEB
/wQEAwIBpjAPBgNVHSUECDAGBgRVHSUAMA8GA1UdEwEB/wQFMAMBAf8wKQYDVR0O
BCIEIHDiJWqgh7/yB1zRFCEhhWMQzcZJbFW0aBA8Avo6dV1JMAoGCCqGSM49BAMC
A0gAMEUCIQDqyiYLs1QNnKjBeAjGzA/xUSru+aK6kmL6hjN0uQZ26wIgZvDjPpdU
XTjPq46iPCqu1l9/XVu7iOIF4blL3I0NN+4=
-----END CERTIFICATE-----`
  },
  orderer_opts: {
    pem: `-----BEGIN CERTIFICATE-----
MIICPDCCAeOgAwIBAgIQcKgU6CLAAHjM4V86MqY7djAKBggqhkjOPQQDAjBwMQsw
CQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVuemhl
bjEZMBcGA1UEChMQYmFua2FsbGlhbmNlLm9yZzEfMB0GA1UEAxMWdGxzY2EuYmFu
a2FsbGlhbmNlLm9yZzAeFw0xODA5MjAxMDE2MDhaFw0yODA5MTcxMDE2MDhaMHAx
CzAJBgNVBAYTAkNOMRIwEAYDVQQIEwlndWFuZ2RvbmcxETAPBgNVBAcTCHNoZW56
aGVuMRkwFwYDVQQKExBiYW5rYWxsaWFuY2Uub3JnMR8wHQYDVQQDExZ0bHNjYS5i
YW5rYWxsaWFuY2Uub3JnMFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAEci43souL
9UZdvCgAJWHKVlN8UcwMTaM63QfU9dKERWW2Hy4KdYs4QRSPUGVGFmlghrPkdDeP
ERT4XvC5/Yx+x6NfMF0wDgYDVR0PAQH/BAQDAgGmMA8GA1UdJQQIMAYGBFUdJQAw
DwYDVR0TAQH/BAUwAwEB/zApBgNVHQ4EIgQgkoB2O9apsjQdtQjvMpRYUvqiqKLD
aTCbIOj3O09g3gcwCgYIKoZIzj0EAwIDRwAwRAIgDZyK1VH+wc1ms76ste0r/1kd
3NFqoXLbQCkeVicFkocCIDqs7HXr8wqWMxI04wIddeNDFD+qlg0sSs/7FPztqN2C
-----END CERTIFICATE-----
`
  },
};
let msp = 'banksMSP';
let certificate = `-----BEGIN CERTIFICATE-----
MIICMDCCAdagAwIBAgIQTPTdIgJ9aaro1sMBW6pWDTAKBggqhkjOPQQDAjB5MQsw
CQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVuemhl
bjEfMB0GA1UEChMWYmFua3MuYmFua2FsbGlhbmNlLm9yZzEiMCAGA1UEAxMZY2Eu
YmFua3MuYmFua2FsbGlhbmNlLm9yZzAeFw0xODA5MjAxMDE2MDhaFw0yODA5MTcx
MDE2MDhaMGwxCzAJBgNVBAYTAkNOMRIwEAYDVQQIEwlndWFuZ2RvbmcxETAPBgNV
BAcTCHNoZW56aGVuMQ8wDQYDVQQLEwZjbGllbnQxJTAjBgNVBAMMHEFkbWluQGJh
bmtzLmJhbmthbGxpYW5jZS5vcmcwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAAQf
JFjUJwRd/gAn2xVi+nGUaZDgf109Rbx2mgIzb4SxzoJj+rPBW+prpxPszH5hd7N2
KJTR8/grZy7nWqfIxgYOo00wSzAOBgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIw
ADArBgNVHSMEJDAigCCRdjIdx4JgUbW35/EWMHAy7jqi5NSQ6mc7hSyHvbTttTAK
BggqhkjOPQQDAgNIADBFAiEA1jEhAgWFuFk/EgC89QPrcJHzb3LV65nL69E5Dltu
NGoCID4zESiXSJ3hwlf1sNVGhJ/uz+f1PVQFhKomB+cs4/IM
-----END CERTIFICATE-----
`;

let private_key = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgUKAqrCuf00FAQM4j
RhwQlXjLJTAmkIo4GpJOkV8HHbKhRANCAAQfJFjUJwRd/gAn2xVi+nGUaZDgf109
Rbx2mgIzb4SxzoJj+rPBW+prpxPszH5hd7N2KJTR8/grZy7nWqfIxgYO
-----END PRIVATE KEY-----
`;

// let peer = ["grpcs://peer0.platform.bankalliance.org:7051"];
//
// let certificate = `-----BEGIN CERTIFICATE-----
// MIICMDCCAdagAwIBAgIQTPTdIgJ9aaro1sMBW6pWDTAKBggqhkjOPQQDAjB5MQsw
// CQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVuemhl
// bjEfMB0GA1UEChMWYmFua3MuYmFua2FsbGlhbmNlLm9yZzEiMCAGA1UEAxMZY2Eu
// YmFua3MuYmFua2FsbGlhbmNlLm9yZzAeFw0xODA5MjAxMDE2MDhaFw0yODA5MTcx
// MDE2MDhaMGwxCzAJBgNVBAYTAkNOMRIwEAYDVQQIEwlndWFuZ2RvbmcxETAPBgNV
// BAcTCHNoZW56aGVuMQ8wDQYDVQQLEwZjbGllbnQxJTAjBgNVBAMMHEFkbWluQGJh
// bmtzLmJhbmthbGxpYW5jZS5vcmcwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAAQf
// JFjUJwRd/gAn2xVi+nGUaZDgf109Rbx2mgIzb4SxzoJj+rPBW+prpxPszH5hd7N2
// KJTR8/grZy7nWqfIxgYOo00wSzAOBgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIw
// ADArBgNVHSMEJDAigCCRdjIdx4JgUbW35/EWMHAy7jqi5NSQ6mc7hSyHvbTttTAK
// BggqhkjOPQQDAgNIADBFAiEA1jEhAgWFuFk/EgC89QPrcJHzb3LV65nL69E5Dltu
// NGoCID4zESiXSJ3hwlf1sNVGhJ/uz+f1PVQFhKomB+cs4/IM
// -----END CERTIFICATE-----
// `;
//
// let private_key = `-----BEGIN PRIVATE KEY-----
// MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgUKAqrCuf00FAQM4j
// RhwQlXjLJTAmkIo4GpJOkV8HHbKhRANCAAQfJFjUJwRd/gAn2xVi+nGUaZDgf109
// Rbx2mgIzb4SxzoJj+rPBW+prpxPszH5hd7N2KJTR8/grZy7nWqfIxgYO
// -----END PRIVATE KEY-----
// `;
//
// let tls_pem = `-----BEGIN CERTIFICATE-----
// MIICVjCCAfygAwIBAgIRAOEcfzFoQFonPveu3+K+E1IwCgYIKoZIzj0EAwIwfDEL
// MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
// ZW4xHzAdBgNVBAoTFmJhbmtzLmJhbmthbGxpYW5jZS5vcmcxJTAjBgNVBAMTHHRs
// c2NhLmJhbmtzLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgwOTIwMTAxNjA4WhcNMjgw
// OTE3MTAxNjA4WjB8MQswCQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREw
// DwYDVQQHEwhzaGVuemhlbjEfMB0GA1UEChMWYmFua3MuYmFua2FsbGlhbmNlLm9y
// ZzElMCMGA1UEAxMcdGxzY2EuYmFua3MuYmFua2FsbGlhbmNlLm9yZzBZMBMGByqG
// SM49AgEGCCqGSM49AwEHA0IABJ3huUWMOaatLphxG2pEn5XCiOYO5QCBYnFLbVUA
// SceWsx4eGn1zt7cEVDBtdCybMnyIAWLR4bOgn3WNm3T/5NCjXzBdMA4GA1UdDwEB
// /wQEAwIBpjAPBgNVHSUECDAGBgRVHSUAMA8GA1UdEwEB/wQFMAMBAf8wKQYDVR0O
// BCIEIHDiJWqgh7/yB1zRFCEhhWMQzcZJbFW0aBA8Avo6dV1JMAoGCCqGSM49BAMC
// A0gAMEUCIQDqyiYLs1QNnKjBeAjGzA/xUSru+aK6kmL6hjN0uQZ26wIgZvDjPpdU
// XTjPq46iPCqu1l9/XVu7iOIF4blL3I0NN+4=
// -----END CERTIFICATE-----`;

let clientProxy = new ClientProxy();
clientProxy.setClientSigningIdentity(private_key, certificate, msp);

describe('=== ClientProxy TestCase ===', function () {
  it("--- installChaincode mycc ---", function (done) {
    clientProxy.installChaincode({url:target_obj.peer_url_1,opts:target_obj.peer_opts}, req.chaincodePath, req.chaincodeId, req.chaincodeVersion, [req.channel_id], req.chaincodeType)
      .then(function (result) {
        console.log("installChaincode  Result:", result[0][0]);
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      });
  });
  it.skip("--- upgradeChaincode mycc---", function (done) {
    clientProxy.instantiateUpgradeChaincode(target_obj, req, 20000)
      .then(function (result) {
        console.log("upgradeChaincode mycc  Result:",result);
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
});


