var assert = require("assert");
var util = require('util');
var should = require('should');
var Promise = require('bluebird');

global.logger = console;

var ClientProxy = require("../lib/ClientProxy.js");

var channelName = 'main';
var chaincodeName = 'mycc';

var targetObj = {
  peer_url : ['grpcs://peer0.platform.bankalliance.org:7051'],
  orderer_url : 'grpcs://orderer0.bankalliance.org:7050',
  event_url : 'grpcs://peer0.platform.bankalliance.org:7053'
};

var certificate = `-----BEGIN CERTIFICATE-----
MIICOTCCAd+gAwIBAgIQOX9lpS/X0pSamLoKDGWUATAKBggqhkjOPQQDAjB/MQsw
CQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVuemhl
bjEiMCAGA1UEChMZcGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzElMCMGA1UEAxMc
Y2EucGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzAeFw0xODA5MTkxMzA5MTRaFw0y
ODA5MTYxMzA5MTRaMG8xCzAJBgNVBAYTAkNOMRIwEAYDVQQIEwlndWFuZ2Rvbmcx
ETAPBgNVBAcTCHNoZW56aGVuMQ8wDQYDVQQLEwZjbGllbnQxKDAmBgNVBAMMH0Fk
bWluQHBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcwWTATBgcqhkjOPQIBBggqhkjO
PQMBBwNCAASfzeQe4P119qtZLCvW4U0dklyWSDYFWu7E2xte+02P4Jj+EcEqD+RH
NHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRWo00wSzAOBgNVHQ8BAf8EBAMCB4AwDAYD
VR0TAQH/BAIwADArBgNVHSMEJDAigCAlAHXV08uwGXy6cYljtLmNHX6uijbq6ymR
2cvazo4pcTAKBggqhkjOPQQDAgNIADBFAiEA5woj3PG2E11xjkrebQY4xnxNq6he
na5Xg4Gq1gafa5wCICKypuUxafdMOlEoCO8ZSnbM+P2D2r54dx37QCPM6ewM
-----END CERTIFICATE-----
`;

var private_key = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgiFmlxXnRvB3otNtC
EPYVySXMoDxT0JwpfHOZtOJH5oOhRANCAASfzeQe4P119qtZLCvW4U0dklyWSDYF
Wu7E2xte+02P4Jj+EcEqD+RHNHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRW
-----END PRIVATE KEY-----
`;


let peerOpts={
  pem:`-----BEGIN CERTIFICATE-----
MIICYjCCAgmgAwIBAgIQWFEJcAFeA/3e6LZQvWQcaTAKBggqhkjOPQQDAjCBgjEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xIjAgBgNVBAoTGXBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcxKDAmBgNVBAMT
H3Rsc2NhLnBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgwOTE5MTMwOTE0
WhcNMjgwOTE2MTMwOTE0WjCBgjELMAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5n
ZG9uZzERMA8GA1UEBxMIc2hlbnpoZW4xIjAgBgNVBAoTGXBsYXRmb3JtLmJhbmth
bGxpYW5jZS5vcmcxKDAmBgNVBAMTH3Rsc2NhLnBsYXRmb3JtLmJhbmthbGxpYW5j
ZS5vcmcwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAAQrbGwcEAU3TOdWjO11PSUh
53TxE2TQp5qV7iDpTAeUrtQGz85XT1j1xMZD70L+amTgAUR+IUj/XZvjXCBDH2K1
o18wXTAOBgNVHQ8BAf8EBAMCAaYwDwYDVR0lBAgwBgYEVR0lADAPBgNVHRMBAf8E
BTADAQH/MCkGA1UdDgQiBCBfccLcRG2uWYzZ+5VOTQRw8C+Rm9VSJM4IwqWEi1cO
hzAKBggqhkjOPQQDAgNHADBEAiBbrlHgSJazj8eWd0S3viUsTuEMOFoXLMyVccPh
HblqOwIgNpTGn1EQ1FOP1d42LejlHipPzMMHHtCYtm/pBTVIf+Q=
-----END CERTIFICATE-----
`
};

var clientProxy = new ClientProxy();
clientProxy.setClientSigningIdentity(private_key,certificate , 'platformMSP');

describe('=== ClientProxy TestCase ===', function () {
  it("--- queryInstalledChaincodes ---", function (done) {
    clientProxy.queryInstalledChaincodes(targetObj.peer_url[0],peerOpts)
      .then(function (result) {
        result.chaincodes.forEach(function (chaincode) {
          chaincode.id=chaincode.id.toString('hex');
        });
        console.log("queryInstalledChaincodes  Result:", result);
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
  it("--- queryInstantiatedChaincodes ---", function (done) {
    clientProxy.queryInstantiatedChaincodes(targetObj.peer_url[0], channelName,peerOpts)
      .then(function (result) {
        console.log("Instantiated Chaincodes  Result:", result)
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
  it("--- queryByChaincode ---", function (done) {
    clientProxy.queryChaincodeByName(targetObj.peer_url[0], channelName, chaincodeName,peerOpts)
      .then(function (result) {
        console.log("queryByChaincode result:", result.toString);
        done();
      })
      .catch(function (err) {
        console.error(err);
        done();
      })
  });
});


