var assert = require("assert");
var util = require('util');
var should = require('should');
var Promise = require('bluebird');
var ByteBuffer = require('bytebuffer');

global.logger = console;

var ClientProxy = require("../lib/ClientProxy.js");

var clientProxy = new ClientProxy();
var channelName = 'main';

var targetObj = {
  peer_url : ['grpc://127.0.0.1:7051'],
  orderer_url : 'grpc://172.16.25.200:7050',
  event_url : 'grpc://172.16.25.200:10053'
};
var blockHash;

var number;


var certificate = `-----BEGIN CERTIFICATE-----
MIICOTCCAd+gAwIBAgIQOX9lpS/X0pSamLoKDGWUATAKBggqhkjOPQQDAjB/MQsw
CQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVuemhl
bjEiMCAGA1UEChMZcGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzElMCMGA1UEAxMc
Y2EucGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzAeFw0xODA5MTkxMzA5MTRaFw0y
ODA5MTYxMzA5MTRaMG8xCzAJBgNVBAYTAkNOMRIwEAYDVQQIEwlndWFuZ2Rvbmcx
ETAPBgNVBAcTCHNoZW56aGVuMQ8wDQYDVQQLEwZjbGllbnQxKDAmBgNVBAMMH0Fk
bWluQHBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcwWTATBgcqhkjOPQIBBggqhkjO
PQMBBwNCAASfzeQe4P119qtZLCvW4U0dklyWSDYFWu7E2xte+02P4Jj+EcEqD+RH
NHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRWo00wSzAOBgNVHQ8BAf8EBAMCB4AwDAYD
VR0TAQH/BAIwADArBgNVHSMEJDAigCAlAHXV08uwGXy6cYljtLmNHX6uijbq6ymR
2cvazo4pcTAKBggqhkjOPQQDAgNIADBFAiEA5woj3PG2E11xjkrebQY4xnxNq6he
na5Xg4Gq1gafa5wCICKypuUxafdMOlEoCO8ZSnbM+P2D2r54dx37QCPM6ewM
-----END CERTIFICATE-----`

var private_key = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgiFmlxXnRvB3otNtC
EPYVySXMoDxT0JwpfHOZtOJH5oOhRANCAASfzeQe4P119qtZLCvW4U0dklyWSDYF
Wu7E2xte+02P4Jj+EcEqD+RHNHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRW
-----END PRIVATE KEY-----`

var peer_opts =  {
  pem: `-----BEGIN CERTIFICATE-----
MIICYjCCAgmgAwIBAgIQWFEJcAFeA/3e6LZQvWQcaTAKBggqhkjOPQQDAjCBgjEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xIjAgBgNVBAoTGXBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcxKDAmBgNVBAMT
H3Rsc2NhLnBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgwOTE5MTMwOTE0
WhcNMjgwOTE2MTMwOTE0WjCBgjELMAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5n
ZG9uZzERMA8GA1UEBxMIc2hlbnpoZW4xIjAgBgNVBAoTGXBsYXRmb3JtLmJhbmth
bGxpYW5jZS5vcmcxKDAmBgNVBAMTH3Rsc2NhLnBsYXRmb3JtLmJhbmthbGxpYW5j
ZS5vcmcwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAAQrbGwcEAU3TOdWjO11PSUh
53TxE2TQp5qV7iDpTAeUrtQGz85XT1j1xMZD70L+amTgAUR+IUj/XZvjXCBDH2K1
o18wXTAOBgNVHQ8BAf8EBAMCAaYwDwYDVR0lBAgwBgYEVR0lADAPBgNVHRMBAf8E
BTADAQH/MCkGA1UdDgQiBCBfccLcRG2uWYzZ+5VOTQRw8C+Rm9VSJM4IwqWEi1cO
hzAKBggqhkjOPQQDAgNHADBEAiBbrlHgSJazj8eWd0S3viUsTuEMOFoXLMyVccPh
HblqOwIgNpTGn1EQ1FOP1d42LejlHipPzMMHHtCYtm/pBTVIf+Q=
-----END CERTIFICATE-----`
}

clientProxy.setClientSigningIdentity(private_key, certificate, 'platformMSP');

describe('=== ClientProxy TestCase ===', function () {
  it("--- getChannelInfo ---", function (done) {
    clientProxy.getChannelInfo(targetObj.peer_url[0], channelName,peer_opts)
      .then(function (result) {
        console.log("Channel Info Result:", result);
        blockHash = result.currentBlockHash;
        number = result.height.toNumber()-1
        console.log('number=',number)
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });

  it.skip("--- queryBlockByHash ---", function (done) {
    clientProxy.queryBlockByHash(targetObj.peer_url[0], channelName, blockHash,peer_opts)
      .then(function (result) {
        console.log("queryBlockByHash Result:", result);
        number = result.header.number;
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
  it("--- queryBlockByNumber ---", function (done) {
    clientProxy.queryBlockByNumber(targetObj.peer_url[0], channelName, number >> 0,peer_opts)
      .then(function (block) {
        console.log("queryBlockByNumber Result:", block);

        var block_num = block.header.number
        var tx_num = block.data.data.legnth

        console.log('>>>block_header:',block.header)

        block.data.data.forEach(function(v,k){
          var channel_header = v.payload.header.channel_header
          var signature_header = v.payload.header.signature_header
          console.log('>>>channel_header:',channel_header)
          console.log('>>>signature_header:',signature_header)

          v.payload.data.actions.forEach(function(action){
            console.log('>>>>>>action_header=',action.header)
            console.log('>>>>>>action_input',clientProxy.decodeChaincodeInvocationSpec(action.payload.chaincode_proposal_payload.input) )
            var input_args = clientProxy.decodeChaincodeInvocationSpec(action.payload.chaincode_proposal_payload.input).chaincode_spec.input.args
            input_args.forEach(function(input_param_buffer,index){
              
              console.log('>>>>>>action_input_args[',index,'] = ',Buffer.from(input_param_buffer.toHex(),'hex').toString('utf8'))
            })
            
            // console.log('>>>>>>action_endorsements',action.payload.action.endorsements)
            console.log('>>>>>>action_events',action.payload.action.proposal_response_payload.extension.events)
            console.log('>>>>>>action_results',action.payload.action.proposal_response_payload.extension.results)

          })
        })



        // console.log('>>>>>',JSON.stringify(block))
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
});


