var assert = require("assert");
var util = require('util');
var should = require('should');
var Promise = require('bluebird');
var ByteBuffer = require('bytebuffer');

global.logger = console;

var ClientProxy = require("../lib/ClientProxy.js");

var clientProxy = new ClientProxy();
var channelName = 'public';

var targetObj = {
  peer_url : ['grpcs://peer0.banks.bankalliance.org:7051'],
  orderer_url : 'grpcs://orderer0.bankalliance.org:7050',
  event_url : 'grpcs://peer0.banks.bankalliance.org:7053'
};
var blockHash;

var number;


var certificate = `-----BEGIN CERTIFICATE-----
MIICVDCCAfqgAwIBAgIRAI959IHJWd3PlYhBHL5ypzMwCgYIKoZIzj0EAwIweTEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xHzAdBgNVBAoTFmJhbmtzLmJhbmthbGxpYW5jZS5vcmcxIjAgBgNVBAMTGWNh
LmJhbmtzLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgxMDE2MDQxNjIxWhcNMTkxMDE2
MDQxNjIxWjB+MQswCQYDVQQGEwJDTjESMBAGA1UECBMJR3Vhbmdkb25nMREwDwYD
VQQHEwhTaGVuemhlbjEOMAwGA1UEChMFYmFua3MxDzANBgNVBAsTBmNsaWVudDEn
MCUGA1UEAwwebWFuYWdlckBiYW5rcy5iYW5rYWxsaWFuY2Uub3JnMFkwEwYHKoZI
zj0CAQYIKoZIzj0DAQcDQgAEIvhjYrUi/ygILnoX8eiFVv8157MlC1Mr0rvkVRP/
W+cCEMnViso9lRv3gJxHETQ4dy8AH/ZI2/tLEkZRjMQx9KNeMFwwDgYDVR0PAQH/
BAQDAgGmMA8GA1UdJQQIMAYGBFUdJQAwDAYDVR0TAQH/BAIwADArBgNVHSMEJDAi
gCAHqjINc/qztSJBnPQQOVrmaIWYg0OPggqMs1Eb173hIzAKBggqhkjOPQQDAgNI
ADBFAiATKLjS8gEVUaCXKUvgKBQFbZLEl+mZ5GNZSQNdjpoXQgIhAMTUYrQHFQFa
Fk/OzkTQXHQup6R3zSbmZinyKHZVSNqd
-----END CERTIFICATE-----`

var private_key = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgqqzTQslPLVfBUTYI
NYRr2yUo6gOFmJ9OtF/QzFBhrA6hRANCAAQi+GNitSL/KAguehfx6IVW/zXnsyUL
UyvSu+RVE/9b5wIQydWKyj2VG/eAnEcRNDh3LwAf9kjb+0sSRlGMxDH0
-----END PRIVATE KEY-----`

var peer_opts =  {
  pem: `-----BEGIN CERTIFICATE-----
MIICVjCCAfygAwIBAgIRAOEcfzFoQFonPveu3+K+E1IwCgYIKoZIzj0EAwIwfDEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xHzAdBgNVBAoTFmJhbmtzLmJhbmthbGxpYW5jZS5vcmcxJTAjBgNVBAMTHHRs
c2NhLmJhbmtzLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgwOTIwMTAxNjA4WhcNMjgw
OTE3MTAxNjA4WjB8MQswCQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREw
DwYDVQQHEwhzaGVuemhlbjEfMB0GA1UEChMWYmFua3MuYmFua2FsbGlhbmNlLm9y
ZzElMCMGA1UEAxMcdGxzY2EuYmFua3MuYmFua2FsbGlhbmNlLm9yZzBZMBMGByqG
SM49AgEGCCqGSM49AwEHA0IABJ3huUWMOaatLphxG2pEn5XCiOYO5QCBYnFLbVUA
SceWsx4eGn1zt7cEVDBtdCybMnyIAWLR4bOgn3WNm3T/5NCjXzBdMA4GA1UdDwEB
/wQEAwIBpjAPBgNVHSUECDAGBgRVHSUAMA8GA1UdEwEB/wQFMAMBAf8wKQYDVR0O
BCIEIHDiJWqgh7/yB1zRFCEhhWMQzcZJbFW0aBA8Avo6dV1JMAoGCCqGSM49BAMC
A0gAMEUCIQDqyiYLs1QNnKjBeAjGzA/xUSru+aK6kmL6hjN0uQZ26wIgZvDjPpdU
XTjPq46iPCqu1l9/XVu7iOIF4blL3I0NN+4=
-----END CERTIFICATE-----`
}

clientProxy.setClientSigningIdentity(private_key, certificate, 'banksMSP');

describe('=== ClientProxy TestCase ===', function () {
  it("--- getChannelInfo ---", function (done) {
    clientProxy.getChannelInfo(targetObj.peer_url[0], channelName,peer_opts)
      .then(function (result) {
        console.log("Channel Info Result:", result);
        blockHash = result.currentBlockHash;
        number = result.height.toNumber()-1
        console.log('number=',number)
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });

  it.skip("--- queryBlockByHash ---", function (done) {
    clientProxy.queryBlockByHash(targetObj.peer_url[0], channelName, blockHash,peer_opts)
      .then(function (result) {
        console.log("queryBlockByHash Result:", result);
        number = result.header.number;
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
  it("--- queryBlockByNumber ---", function (done) {
    clientProxy.queryBlockByNumber(targetObj.peer_url[0], channelName, number >> 0,peer_opts)
      .then(function (block) {
        console.log("queryBlockByNumber Result:", block);

        var block_num = block.header.number
        var tx_num = block.data.data.legnth

        console.log('>>>block_header:',block.header)

        block.data.data.forEach(function(v,k){
          var channel_header = v.payload.header.channel_header
          var signature_header = v.payload.header.signature_header
          console.log('>>>channel_header:',channel_header)
          console.log('>>>signature_header:',signature_header)

          v.payload.data.actions.forEach(function(action){
            console.log('>>>>>>action_header=',action.header)
            console.log('>>>>>>action_input',clientProxy.decodeChaincodeInvocationSpec(action.payload.chaincode_proposal_payload.input) )
            var input_args = clientProxy.decodeChaincodeInvocationSpec(action.payload.chaincode_proposal_payload.input).chaincode_spec.input.args
            input_args.forEach(function(input_param_buffer,index){
              
              console.log('>>>>>>action_input_args[',index,'] = ',Buffer.from(input_param_buffer.toHex(),'hex').toString('utf8'))
            })
            
            // console.log('>>>>>>action_endorsements',action.payload.action.endorsements)
            console.log('>>>>>>action_events',action.payload.action.proposal_response_payload.extension.events)
            console.log('>>>>>>action_results',action.payload.action.proposal_response_payload.extension.results)

          })
        })



        // console.log('>>>>>',JSON.stringify(block))
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
});


