var assert = require("assert");
var should = require('should');
var Promise = require('bluebird')

global.logger = console

var ClientProxy = require("../lib/ClientProxy.js");

var timeStamp = Math.floor(Date.now()/1000)

var clientProxy = new ClientProxy()

var channel_id = 'main' 
var chaincodeName = 'mycc'
var fcn = 'wCreate'
var args = ['{"tokenId":"T'+timeStamp+'","owner":"深圳区块链金融服务有限公司","type":"DEBIT_RECEIVE","balance":10000000.80,"createTime":'+timeStamp+'}']
var peer_url = 'grpc://127.0.0.1:7051'
var orderer_url = 'grpc://127.0.0.1:7050'
var event_url = 'grpc://127.0.0.1:7053'

var certificate = `-----BEGIN CERTIFICATE-----
MIICOTCCAd+gAwIBAgIQOX9lpS/X0pSamLoKDGWUATAKBggqhkjOPQQDAjB/MQsw
CQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVuemhl
bjEiMCAGA1UEChMZcGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzElMCMGA1UEAxMc
Y2EucGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzAeFw0xODA5MTkxMzA5MTRaFw0y
ODA5MTYxMzA5MTRaMG8xCzAJBgNVBAYTAkNOMRIwEAYDVQQIEwlndWFuZ2Rvbmcx
ETAPBgNVBAcTCHNoZW56aGVuMQ8wDQYDVQQLEwZjbGllbnQxKDAmBgNVBAMMH0Fk
bWluQHBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcwWTATBgcqhkjOPQIBBggqhkjO
PQMBBwNCAASfzeQe4P119qtZLCvW4U0dklyWSDYFWu7E2xte+02P4Jj+EcEqD+RH
NHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRWo00wSzAOBgNVHQ8BAf8EBAMCB4AwDAYD
VR0TAQH/BAIwADArBgNVHSMEJDAigCAlAHXV08uwGXy6cYljtLmNHX6uijbq6ymR
2cvazo4pcTAKBggqhkjOPQQDAgNIADBFAiEA5woj3PG2E11xjkrebQY4xnxNq6he
na5Xg4Gq1gafa5wCICKypuUxafdMOlEoCO8ZSnbM+P2D2r54dx37QCPM6ewM
-----END CERTIFICATE-----`

var private_key = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgiFmlxXnRvB3otNtC
EPYVySXMoDxT0JwpfHOZtOJH5oOhRANCAASfzeQe4P119qtZLCvW4U0dklyWSDYF
Wu7E2xte+02P4Jj+EcEqD+RHNHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRW
-----END PRIVATE KEY-----`

var peer_opts =  {
  pem: `-----BEGIN CERTIFICATE-----
MIICYjCCAgmgAwIBAgIQWFEJcAFeA/3e6LZQvWQcaTAKBggqhkjOPQQDAjCBgjEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xIjAgBgNVBAoTGXBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcxKDAmBgNVBAMT
H3Rsc2NhLnBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgwOTE5MTMwOTE0
WhcNMjgwOTE2MTMwOTE0WjCBgjELMAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5n
ZG9uZzERMA8GA1UEBxMIc2hlbnpoZW4xIjAgBgNVBAoTGXBsYXRmb3JtLmJhbmth
bGxpYW5jZS5vcmcxKDAmBgNVBAMTH3Rsc2NhLnBsYXRmb3JtLmJhbmthbGxpYW5j
ZS5vcmcwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAAQrbGwcEAU3TOdWjO11PSUh
53TxE2TQp5qV7iDpTAeUrtQGz85XT1j1xMZD70L+amTgAUR+IUj/XZvjXCBDH2K1
o18wXTAOBgNVHQ8BAf8EBAMCAaYwDwYDVR0lBAgwBgYEVR0lADAPBgNVHRMBAf8E
BTADAQH/MCkGA1UdDgQiBCBfccLcRG2uWYzZ+5VOTQRw8C+Rm9VSJM4IwqWEi1cO
hzAKBggqhkjOPQQDAgNHADBEAiBbrlHgSJazj8eWd0S3viUsTuEMOFoXLMyVccPh
HblqOwIgNpTGn1EQ1FOP1d42LejlHipPzMMHHtCYtm/pBTVIf+Q=
-----END CERTIFICATE-----`
}
var orderer_opts = {
  pem: `-----BEGIN CERTIFICATE-----
MIICPTCCAeSgAwIBAgIRAP0eBcrG0hY6RkldTWDbGzEwCgYIKoZIzj0EAwIwcDEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xGTAXBgNVBAoTEGJhbmthbGxpYW5jZS5vcmcxHzAdBgNVBAMTFnRsc2NhLmJh
bmthbGxpYW5jZS5vcmcwHhcNMTgwOTE5MTMwOTE1WhcNMjgwOTE2MTMwOTE1WjBw
MQswCQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVu
emhlbjEZMBcGA1UEChMQYmFua2FsbGlhbmNlLm9yZzEfMB0GA1UEAxMWdGxzY2Eu
YmFua2FsbGlhbmNlLm9yZzBZMBMGByqGSM49AgEGCCqGSM49AwEHA0IABHAkTu6O
gn49yRFz8dR81WGlWiDEpmmu1CuzUvtk+Q5fUTDvpQQKGcHnkhDHGQfLXRN+9QEa
msWJ7oQO6YkdAU6jXzBdMA4GA1UdDwEB/wQEAwIBpjAPBgNVHSUECDAGBgRVHSUA
MA8GA1UdEwEB/wQFMAMBAf8wKQYDVR0OBCIEIF3zrholATtsoNUvG/5OtKy18604
cGjUUtVJd4PUtCjMMAoGCCqGSM49BAMCA0cAMEQCICXeROFCLRhceGCEl47niNFr
ypziNPNWa2rP0foXEm4LAiBBu0kAftOHD4vGD2PdVrqgtmBwj0kmD2IwnuKW74sS
pg==
-----END CERTIFICATE-----`
}

clientProxy.setUserIdentity(certificate,'platformMSP')
clientProxy.setAdminSigningIdentity(private_key,certificate,'platformMSP')

describe('=== ClientProxy TestCase ===', function(){
  /*
  it("--- getKey() ---",function(done){
    var key = FabricHelper.getKey(private_key)
    console.log('key=',key)
    
    done()
  })

  it("--- newIdentity(创建identity) ---",function(done){
    var identity = FabricHelper.newIdentity(certificate,'platformMSP')
    console.log('identity=',identity)
    var tx_id = FabricHelper.newTransactionID(identity)
    console.log('tx_id=',tx_id)

    console.log('identity.serialize()=',identity.serialize())
    done()
  })

  it("--- buildTransactionProposal(创建未签名的proposal) ---",function(done){
  	var proposal = fabricClient.buildTransactionProposal(certificate,channel_id,chaincodeName,fcn,args)
    console.log("proposal=",proposal)
    done()
  })
*/


  /*it.skip("--- signProposal and sendPeersProposal(query) ---",function(done){
    var proposal = fabricHelper.buildTransactionProposal(channel_id,chaincodeName,fcn,args)
    var proposal_bytes = proposal.toBuffer()
    // console.log("proposal=",proposal)

    var proposal_decode = fabricHelper.decodeProposal(proposal_bytes)
    // console.log("proposal_decode=",JSON.stringify(proposal_decode))

    
    // console.log("channel_header=",proposal_decode.header.channel_header)

    var proposal_signature = fabricHelper.sign(private_key,proposal_bytes)
    // console.log("proposal_signature=",proposal_signature)
    var verifyResult = fabricHelper.verifyProposal(proposal_signature,proposal_bytes)
    // console.log("verifyResult=",verifyResult)
   
    var context = {}
    fabricHelper.sendPeersProposal(peer_url,proposal_bytes,proposal_signature,20000)
    .then(function(responses){
      return fabricHelper.decodeProposalResponse(responses)
    })
    .then(function(responses){
      context.chaincode_result = responses[0].chaincode_result
      context.proposalResponses = responses
      if(fabricHelper.hasWriteSet(responses)){
        var tx_payload_bytes =  fabricHelper.buildTransactionPayload(proposal_bytes,responses)
        return Promise.resolve(tx_payload_bytes)
      }else{
        return Promise.resolve(null)
      }
      
      
    })
    .then(function(tx_payload_bytes){
      if(tx_payload_bytes){
        context.tx_payload = tx_payload_bytes.toString('hex')
      }
      console.log(">>>>>>result=",context)
      done()
    })
    .catch(function(err){
      console.error(err)
      done()
    })
  })*/



  it("--- signProposal and sendPeersProposal(invoke) ---",function(done){
    var proposal = clientProxy.buildTransactionProposal(channel_id,chaincodeName,fcn,args)
    var proposal_bytes = proposal.toBuffer()
     // console.log("proposal=",proposal)

    var proposal_decode = clientProxy.decodeProposal(proposal_bytes)
     // console.log("proposal_decode=",JSON.stringify(proposal_decode))

    
     // console.log("channel_header=",proposal_decode.header.channel_header)

    var proposal_signature = clientProxy.sign(private_key,proposal_bytes)
    // console.log("proposal_signature=",proposal_signature)
    var verifyResult = clientProxy.verifyProposal(proposal_signature,proposal_bytes)
    // console.log("verifyResult=",verifyResult)
   
    var context = {}
    clientProxy.sendPeersProposal(peer_url,proposal_bytes,proposal_signature,20000,peer_opts)
    .then(function(responses){
      return clientProxy.decodeProposalResponse(responses)
    })
    .then(function(responses){
      context.chaincode_result = responses[0].chaincode_result
      context.proposalResponses = responses
      if(clientProxy.hasWriteSet(responses)){
        var tx_payload_bytes =  clientProxy.buildTransactionPayload(proposal_bytes,responses)
        return Promise.resolve(tx_payload_bytes)
      }else{
        return Promise.resolve(null)
      }
      
      
    })
    .then(function(tx_payload_bytes){
      if(tx_payload_bytes){
        context.tx_payload = tx_payload_bytes.toString('hex')
      }

      tx_signature = clientProxy.sign(private_key,tx_payload_bytes)
      
      return clientProxy.sendTransaction({
        orderer_url:orderer_url,
        event_url:event_url,
        orderer_opts: orderer_opts,
        peer_opts: peer_opts
      },
        tx_payload_bytes,tx_signature,50000)
    })
    .then(function(tx_result){
      context.tx_result = tx_result
      console.log("context=",context)
      return clientProxy.queryTransaction(peer_url,channel_id,tx_result.tx_id,peer_opts)
      
    })
    .then(function(tx_receipt){
      console.log('tx_receipt=',JSON.stringify(tx_receipt))

      return clientProxy.getChannelInfo(peer_url,channel_id,peer_opts)
      
    })
    .then(function(channel_info){
      console.log('channel_info=',channel_info)
      done()
    })
    .catch(function(err){
      console.error(err)
      done()
    })

  })



});


