var assert = require("assert");
var util = require('util');
var should = require('should');
var Promise = require('bluebird');
var ByteBuffer = require('bytebuffer');

global.logger = console;

var ClientProxy = require("../lib/ClientProxy.js");

var clientProxy = new ClientProxy();
var channelName = 'public';


var blockHash;

var number;


var certificate = `-----BEGIN CERTIFICATE-----
MIICSDCCAe+gAwIBAgIRAOnoolmkEl1WsqHiYtjU7EEwCgYIKoZIzj0EAwIweTEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xHzAdBgNVBAoTFmJhbmtzLmJhbmthbGxpYW5jZS5vcmcxIjAgBgNVBAMTGWNh
LmJhbmtzLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgxMTI3MDczNDI3WhcNMTkxMTI3
MDczNDI3WjBzMQswCQYDVQQGEwJDTjESMBAGA1UECAwJ5bm/5Lic55yBMRIwEAYD
VQQHDAnmt7HlnLPluIIxFTATBgNVBAoMDOaYk+mTvuenkeaKgDEPMA0GA1UECxMG
Y2xpZW50MRQwEgYDVQQDEwsxMzA4ODg3MDkxMDBZMBMGByqGSM49AgEGCCqGSM49
AwEHA0IABHgDHRPazS+U99eYzAWohyrmn9/yAFjQBRSApnjrfdExiBrJpeilWIWo
UNq3+Yo7+5URAR/hSWZJgufK33KWBOOjXjBcMA4GA1UdDwEB/wQEAwIBpjAPBgNV
HSUECDAGBgRVHSUAMAwGA1UdEwEB/wQCMAAwKwYDVR0jBCQwIoAgkXYyHceCYFG1
t+fxFjBwMu46ouTUkOpnO4Ush7207bUwCgYIKoZIzj0EAwIDRwAwRAIgVYLwrajB
83NFKQ4OMYRIH/DQyIH/UqStUbin63tOt7ACIDet6f84rAfQ+cwB+0Rcj0sQZMPk
oS1AhsC4dWRP6M+Z
-----END CERTIFICATE-----`

var private_key = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgRmcxsWTVFLf/Jd6S
zy3PBznHRuGFXRMTfz1/MSLpi0+hRANCAAR4Ax0T2s0vlPfXmMwFqIcq5p/f8gBY
0AUUgKZ4633RMYgayaXopViFqFDat/mKO/uVEQEf4UlmSYLnyt9ylgTj
-----END PRIVATE KEY-----`

var peer_opts =  {
  pem: `-----BEGIN CERTIFICATE-----
MIICVjCCAfygAwIBAgIRAOEcfzFoQFonPveu3+K+E1IwCgYIKoZIzj0EAwIwfDEL
MAkGA1UEBhMCQ04xEjAQBgNVBAgTCWd1YW5nZG9uZzERMA8GA1UEBxMIc2hlbnpo
ZW4xHzAdBgNVBAoTFmJhbmtzLmJhbmthbGxpYW5jZS5vcmcxJTAjBgNVBAMTHHRs
c2NhLmJhbmtzLmJhbmthbGxpYW5jZS5vcmcwHhcNMTgwOTIwMTAxNjA4WhcNMjgw
OTE3MTAxNjA4WjB8MQswCQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREw
DwYDVQQHEwhzaGVuemhlbjEfMB0GA1UEChMWYmFua3MuYmFua2FsbGlhbmNlLm9y
ZzElMCMGA1UEAxMcdGxzY2EuYmFua3MuYmFua2FsbGlhbmNlLm9yZzBZMBMGByqG
SM49AgEGCCqGSM49AwEHA0IABJ3huUWMOaatLphxG2pEn5XCiOYO5QCBYnFLbVUA
SceWsx4eGn1zt7cEVDBtdCybMnyIAWLR4bOgn3WNm3T/5NCjXzBdMA4GA1UdDwEB
/wQEAwIBpjAPBgNVHSUECDAGBgRVHSUAMA8GA1UdEwEB/wQFMAMBAf8wKQYDVR0O
BCIEIHDiJWqgh7/yB1zRFCEhhWMQzcZJbFW0aBA8Avo6dV1JMAoGCCqGSM49BAMC
A0gAMEUCIQDqyiYLs1QNnKjBeAjGzA/xUSru+aK6kmL6hjN0uQZ26wIgZvDjPpdU
XTjPq46iPCqu1l9/XVu7iOIF4blL3I0NN+4=
-----END CERTIFICATE-----`
}

var targetObj = {
  peer_url : ['grpcs://peer0.banks.bankalliance.org:7051'],
  orderer_url : 'grpcs://orderer0.bankalliance.org:7050',
  event_url : 'grpcs://peer0.banks.bankalliance.org:7053',
  peer_opts: peer_opts
};


clientProxy.setClientSigningIdentity(private_key, certificate, 'banksMSP');

describe('=== ClientProxy TestCase ===', function () {
  it("--- getChannelInfo ---", function (done) {
    clientProxy.getChannelInfo(targetObj.peer_url[0], channelName,peer_opts)
      .then(function (result) {
        console.log("Channel Info Result:", result);
        blockHash = result.currentBlockHash;
        number = result.height.toNumber()-1
        console.log('number=',number)
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });

  it.skip("--- queryBlockByHash ---", function (done) {
    clientProxy.queryBlockByHash(targetObj.peer_url[0], channelName, blockHash,peer_opts)
      .then(function (result) {
        console.log("queryBlockByHash Result:", result);
        number = result.header.number;
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
  it("--- queryBlockByNumber ---", function (done) {
    clientProxy.queryBlockByNumber(targetObj.peer_url[0], channelName, number >> 0,peer_opts)
      .then(function (block) {
        console.log("queryBlockByNumber Result:", block);

        var block_num = block.header.number
        var tx_num = block.data.data.legnth

        console.log('>>>block_header:',block.header)

        block.data.data.forEach(function(v,k){
          var channel_header = v.payload.header.channel_header
          var signature_header = v.payload.header.signature_header
          console.log('>>>channel_header:',channel_header)
          console.log('>>>signature_header:',signature_header)

          v.payload.data.actions.forEach(function(action){
            console.log('>>>>>>action_header=',action.header)
            console.log('>>>>>>action_input',clientProxy.decodeChaincodeInvocationSpec(action.payload.chaincode_proposal_payload.input) )
            var input_args = clientProxy.decodeChaincodeInvocationSpec(action.payload.chaincode_proposal_payload.input).chaincode_spec.input.args
            input_args.forEach(function(input_param_buffer,index){
              
              console.log('>>>>>>action_input_args[',index,'] = ',Buffer.from(input_param_buffer.toHex(),'hex').toString('utf8'))
            })
            
            // console.log('>>>>>>action_endorsements',action.payload.action.endorsements)
            console.log('>>>>>>action_events',action.payload.action.proposal_response_payload.extension.events)
            console.log('>>>>>>action_results',action.payload.action.proposal_response_payload.extension.results)

          })
        })



        // console.log('>>>>>',JSON.stringify(block))
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
});


