const assert = require("assert");
const util = require('util');
const should = require('should');
const Promise = require('bluebird');

global.logger = console;

const ClientProxy = require("../lib/ClientProxy.js");

let req = {
  channel_id: 'main',
  chaincodeId: 'mycc',
  chaincodeVersion: "1.0",
  chaincodeType: 'golang',
  chaincodePath: "chaincode/supplychain",
  isUpdate: false
};
let target_obj = {
  peer_url: "grpc://peer0.platform.bankalliance.org:7051",
  orderer_url: "grpc://orderer0.bankalliance.org:7050",
  event_url: "grpc://peer1.banks.bankalliance.org:7053",
};
let msp = 'platformMSP';

var certificate = `-----BEGIN CERTIFICATE-----
MIICOTCCAd+gAwIBAgIQOX9lpS/X0pSamLoKDGWUATAKBggqhkjOPQQDAjB/MQsw
CQYDVQQGEwJDTjESMBAGA1UECBMJZ3Vhbmdkb25nMREwDwYDVQQHEwhzaGVuemhl
bjEiMCAGA1UEChMZcGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzElMCMGA1UEAxMc
Y2EucGxhdGZvcm0uYmFua2FsbGlhbmNlLm9yZzAeFw0xODA5MTkxMzA5MTRaFw0y
ODA5MTYxMzA5MTRaMG8xCzAJBgNVBAYTAkNOMRIwEAYDVQQIEwlndWFuZ2Rvbmcx
ETAPBgNVBAcTCHNoZW56aGVuMQ8wDQYDVQQLEwZjbGllbnQxKDAmBgNVBAMMH0Fk
bWluQHBsYXRmb3JtLmJhbmthbGxpYW5jZS5vcmcwWTATBgcqhkjOPQIBBggqhkjO
PQMBBwNCAASfzeQe4P119qtZLCvW4U0dklyWSDYFWu7E2xte+02P4Jj+EcEqD+RH
NHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRWo00wSzAOBgNVHQ8BAf8EBAMCB4AwDAYD
VR0TAQH/BAIwADArBgNVHSMEJDAigCAlAHXV08uwGXy6cYljtLmNHX6uijbq6ymR
2cvazo4pcTAKBggqhkjOPQQDAgNIADBFAiEA5woj3PG2E11xjkrebQY4xnxNq6he
na5Xg4Gq1gafa5wCICKypuUxafdMOlEoCO8ZSnbM+P2D2r54dx37QCPM6ewM
-----END CERTIFICATE-----`

var private_key = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQgiFmlxXnRvB3otNtC
EPYVySXMoDxT0JwpfHOZtOJH5oOhRANCAASfzeQe4P119qtZLCvW4U0dklyWSDYF
Wu7E2xte+02P4Jj+EcEqD+RHNHPgIEEDR4Y3Uv0GCq+kYPt5I9CZIsRW
-----END PRIVATE KEY-----`



let clientProxy = new ClientProxy();
clientProxy.setClientSigningIdentity(private_key, certificate, msp);

describe('=== ClientProxy TestCase ===', function () {
  it("--- installChaincode mycc ---", function (done) {
    clientProxy.installChaincode(target_obj.peer_url, req.chaincodePath, req.chaincodeId, req.chaincodeVersion, [req.channel_id], req.chaincodeType, target_obj.peer_opts)
      .then(function (result) {
        console.log("installChaincode  Result:", result[0][0]);
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      });
  });
  it.skip("--- upgradeChaincode mycc---", function (done) {
    clientProxy.instantiateUpgradeChaincode(target_obj, req, 20000)
      .then(function (result) {
        console.log("upgradeChaincode mycc  Result:",result);
        done()
      })
      .catch(function (err) {
        console.error(err);
        done()
      })
  });
});


